"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.format = exports.ap = exports.map = exports.split = exports.make = exports.addWarning = exports.warning = exports.of = exports.warn_ = exports.URI = void 0;
const wrapAnsi = require("wrap-ansi");
exports.URI = 'Warn';
exports.warn_ = {
    URI: exports.URI,
    map: (ma, f) => ({
        payload: f(ma.payload),
        warnings: ma.warnings,
    }),
    ap: (fab, fa) => ({
        payload: fab.payload(fa.payload),
        warnings: fab.warnings.concat(fa.warnings),
    }),
    of: of,
    reduce: (fa, b, f) => f(b, fa.payload),
    reduceRight: (fa, b, f) => f(fa.payload, b),
    foldMap: (_M) => (fa, f) => f(fa.payload),
    traverse: (F) => (ta, f) => F.map(f(ta.payload), (b) => ({ payload: b, warnings: ta.warnings })),
    sequence: (F) => (ta) => F.map(ta.payload, (a) => ({ payload: a, warnings: ta.warnings })),
};
function of(payload) {
    return { payload, warnings: [] };
}
exports.of = of;
function warning(payload, summary, description = '') {
    return {
        payload,
        warnings: [{ summary, description }],
    };
}
exports.warning = warning;
function addWarning(summary, description) {
    return (warn) => {
        return {
            payload: warn.payload,
            warnings: [...warn.warnings, { summary, description }],
        };
    };
}
exports.addWarning = addWarning;
function make(payload, warnings) {
    return { payload, warnings };
}
exports.make = make;
function split(warn) {
    return [warn.payload, warn.warnings];
}
exports.split = split;
function map(f) {
    return (a) => {
        return Object.assign(Object.assign({}, a), { payload: f(a.payload) });
    };
}
exports.map = map;
function ap(fa) {
    return (fab) => exports.warn_.ap(fab, fa);
}
exports.ap = ap;
function format(warnings, verbose, columns) {
    let result = warnings
        .map((warning) => `
WARNING: ${warning.summary}${verbose && warning.description != ''
        ? '\n\n' + warning.description
        : ''}`)
        .join('\n');
    result += `

Due to the problems listed above, the inferred types may be inaccurate with respect to nullability.
`;
    if (verbose) {
        result += `
Please open an issue on https://github.com/akheron/sqltyper.

Include the above error message, relevant parts of your database schema (CREATE TABLE statements, CREATE TYPE statements, etc.) and the SQL statement that failed to parse.

Thank you in advance!
`;
    }
    else {
        result += `
Re-run with --verbose for instructions on how to report or fix this.
`;
    }
    return wrapAnsi(result.trim(), columns, { hard: true, trim: false });
}
exports.format = format;
