"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.clearCache = exports.disconnect = exports.clients = exports.connect = void 0;
const Either = require("fp-ts/lib/Either");
const postgres = require("./postgres");
const schema_1 = require("./schema");
const tstype_1 = require("./tstype");
function connect(connectionString) {
    return __awaiter(this, void 0, void 0, function* () {
        const postgresClient = connectionString
            ? postgres(connectionString)
            : postgres();
        return Either.right(yield clients(postgresClient));
    });
}
exports.connect = connect;
function clients(postgresClient) {
    return __awaiter(this, void 0, void 0, function* () {
        const schema = (0, schema_1.schemaClient)(postgresClient);
        const types = yield (0, tstype_1.typeClient)(postgresClient);
        return { postgres: postgresClient, schema, types };
    });
}
exports.clients = clients;
function disconnect(clients) {
    return __awaiter(this, void 0, void 0, function* () {
        yield clients.postgres.end({ timeout: 5 });
    });
}
exports.disconnect = disconnect;
function clearCache(clients) {
    return __awaiter(this, void 0, void 0, function* () {
        // The type client caches stuff about user-defined SQL types.
        // Recreate it to clear the cache.
        const types = yield (0, tstype_1.typeClient)(clients.postgres);
        return Object.assign(Object.assign({}, clients), { types });
    });
}
exports.clearCache = clearCache;
