"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.builtinFunctionNullSafety = exports.findBuiltinFunction = exports.operatorNullSafety = exports.isOperatorCommutative = exports.findOperator = void 0;
const Option = require("fp-ts/lib/Option");
const function_1 = require("fp-ts/lib/function");
const constants_1 = require("./constants");
function findOperator(name) {
    const upCaseName = name.toUpperCase();
    return Option.fromNullable(constants_1.operators.find((op) => op.op === upCaseName));
}
exports.findOperator = findOperator;
function isOperatorCommutative(name) {
    return (0, function_1.pipe)(findOperator(name), Option.map((op) => op.commutative || false), Option.getOrElse(() => false));
}
exports.isOperatorCommutative = isOperatorCommutative;
function operatorNullSafety(name) {
    return (0, function_1.pipe)(findOperator(name), Option.map((op) => op.nullSafety), Option.toNullable);
}
exports.operatorNullSafety = operatorNullSafety;
function findBuiltinFunction(name) {
    const downCaseName = name.toLowerCase();
    return Option.fromNullable(constants_1.builtinFunctions.find((f) => f.name === downCaseName));
}
exports.findBuiltinFunction = findBuiltinFunction;
function builtinFunctionNullSafety(name) {
    return (0, function_1.pipe)(findBuiltinFunction(name), Option.map((func) => func.nullSafety), Option.toNullable);
}
exports.builtinFunctionNullSafety = builtinFunctionNullSafety;
