"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.indexModuleTS = exports.generateTSCode = exports.sqlToStatementDescription = void 0;
const Task = require("fp-ts/lib/Task");
const TaskEither = require("fp-ts/lib/TaskEither");
const function_1 = require("fp-ts/lib/function");
const codegen_1 = require("./codegen");
const describe_1 = require("./describe");
const infer_1 = require("./infer");
const preprocess_1 = require("./preprocess");
const prettify_1 = require("./prettify");
function sqlToStatementDescription(clients, sql) {
    return (0, function_1.pipe)(Task.of(sql), Task.map(preprocess_1.preprocessSQL), TaskEither.chain((processed) => (0, describe_1.describeStatement)(clients.postgres, processed.sql, processed.paramNames)), TaskEither.chain((stmt) => Task.of((0, codegen_1.validateStatement)(stmt))), TaskEither.chain((stmt) => TaskEither.rightTask((0, infer_1.inferStatementNullability)(clients.schema, stmt))));
}
exports.sqlToStatementDescription = sqlToStatementDescription;
function generateTSCode(clients, sourceFileName, stmt, funcName, options) {
    const { prettierFileName = null, target = 'pg', module = 'pg', } = options || {};
    return (0, function_1.pipe)(TaskEither.right(stmt), TaskEither.chain((stmt) => TaskEither.rightTask((0, codegen_1.generateTypeScript)(clients.types, sourceFileName, target, module, funcName, stmt))), TaskEither.chain((tsCode) => prettierFileName != null
        ? TaskEither.rightTask(() => (0, prettify_1.runPrettier)(prettierFileName, tsCode))
        : TaskEither.right(tsCode)));
}
exports.generateTSCode = generateTSCode;
function indexModuleTS(dirPath, nestedDirs, tsModules, options) {
    const { prettierFileName = null } = options || {};
    return (0, function_1.pipe)(Task.of((0, codegen_1.generateIndexModule)(dirPath, nestedDirs, tsModules)), Task.chain((tsCode) => prettierFileName != null
        ? () => (0, prettify_1.runPrettier)(prettierFileName, tsCode)
        : Task.of(tsCode)));
}
exports.indexModuleTS = indexModuleTS;
