"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.parse = exports.parenthesized = void 0;
const Either_1 = require("fp-ts/lib/Either");
const typed_parser_1 = require("../typed-parser");
const ast_1 = require("../ast");
const utils_1 = require("./utils");
const token_1 = require("./token");
const special_functions_1 = require("./special-functions");
// ( ... )
function parenthesized(parser) {
    return (0, typed_parser_1.seq2)((0, token_1.symbol)('('), parser, (0, token_1.symbol)(')'));
}
exports.parenthesized = parenthesized;
// [ AS ] identifier
const as = (0, typed_parser_1.seq2)((0, utils_1.optional)((0, token_1.reservedWord)('AS')), token_1.identifier);
// AS identifier
const reqAs = (0, typed_parser_1.seq2)((0, token_1.reservedWord)('AS'), token_1.identifier);
// [ schema . ] table
const tableRef = (0, typed_parser_1.seq)(token_1.identifier, (0, utils_1.optional)((0, typed_parser_1.seq2)((0, token_1.symbol)('.'), token_1.identifier)))((id1, id2) => (id2 ? ast_1.TableRef.create(id1, id2) : ast_1.TableRef.create(null, id1)));
// Expressions
const arraySubQueryExpr = (0, typed_parser_1.seq)((0, token_1.reservedWord)('ARRAY'), parenthesized((0, typed_parser_1.lazy)(() => select)))((_arr, subquery) => ast_1.Expression.createArraySubQuery(subquery));
const caseBranch = (0, typed_parser_1.seq)((0, token_1.reservedWord)('WHEN'), (0, typed_parser_1.lazy)(() => expression), (0, token_1.reservedWord)('THEN'), (0, typed_parser_1.lazy)(() => expression))((_when, condition, _then, result) => ({ condition, result }));
const caseElse = (0, typed_parser_1.seq2)((0, token_1.reservedWord)('ELSE'), (0, typed_parser_1.lazy)(() => expression));
const caseExpr = (0, typed_parser_1.seq)((0, token_1.reservedWord)('CASE'), caseBranch, (0, typed_parser_1.many)(caseBranch), (0, utils_1.optional)(caseElse), (0, token_1.reservedWord)('END'))((_case, branch1, branches, else_, _end) => ast_1.Expression.createCase([branch1, ...branches], else_));
const functionArguments = parenthesized((0, typed_parser_1.oneOf)(
// func(*) means no arguments
(0, typed_parser_1.seqConst)([], (0, token_1.symbol)('*')), (0, typed_parser_1.sepBy)((0, token_1.symbol)(','), (0, typed_parser_1.lazy)(() => expression))));
const windowDefinition = (0, typed_parser_1.oneOf)(token_1.identifier, (0, typed_parser_1.seq)((0, utils_1.optional)((0, typed_parser_1.seq3)((0, token_1.reservedWord)('PARTITION'), (0, token_1.reservedWord)('BY'), (0, typed_parser_1.lazy)(() => expression))), (0, utils_1.optional)((0, typed_parser_1.lazy)(() => orderBy)))((partition, order) => ({
    partitionBy: partition,
    orderBy: order,
})));
const columnRefOrFunctionCallExpr = (0, typed_parser_1.seq)(token_1.identifier, (0, utils_1.optional)((0, typed_parser_1.seq2)((0, token_1.symbol)('.'), token_1.identifier)), (0, utils_1.optional)((0, typed_parser_1.seq)(functionArguments, (0, utils_1.optional)((0, typed_parser_1.seq2)((0, token_1.reservedWord)('FILTER'), parenthesized((0, typed_parser_1.seq2)((0, token_1.reservedWord)('WHERE'), (0, typed_parser_1.lazy)(() => expression))))), (0, utils_1.optional)((0, typed_parser_1.seq2)((0, token_1.reservedWord)('OVER'), (0, typed_parser_1.oneOf)(token_1.identifier, parenthesized(windowDefinition)))))((argList, filter, window) => [argList, filter, window])))((ident, ident2, fnCall) => {
    if (fnCall) {
        const [argList, filter, window] = fnCall;
        if (ident2)
            return ast_1.Expression.createFunctionCall(ident, ident2, argList, filter, window);
        return ast_1.Expression.createFunctionCall(null, ident, argList, filter, window);
    }
    if (ident2) {
        return ast_1.Expression.createTableColumnRef(ident, ident2);
    }
    return ast_1.Expression.createColumnRef(ident);
});
const strEscape = (0, typed_parser_1.seq2)((0, token_1.symbolKeepWS)('\\'), (0, typed_parser_1.oneOf)((0, typed_parser_1.keyword)("'", "'"), (0, typed_parser_1.keyword)('\\', '\\'), (0, typed_parser_1.keyword)('/', '/'), (0, typed_parser_1.keyword)('b', '\b'), (0, typed_parser_1.keyword)('f', '\f'), (0, typed_parser_1.keyword)('n', '\n'), (0, typed_parser_1.keyword)('r', '\r'), (0, typed_parser_1.keyword)('t', '\t')));
const strInner = (0, typed_parser_1.seq)((0, typed_parser_1.stringBefore)("[\\']"), (0, typed_parser_1.oneOf)((0, typed_parser_1.seq)(strEscape, (0, typed_parser_1.lazy)(() => strInner))((e, t) => e + t), (0, typed_parser_1.constant)('')))((s, tail) => s + tail);
const stringConstant = (0, typed_parser_1.seq2)((0, token_1.symbolKeepWS)("'"), strInner, (0, token_1.symbol)("'"));
const constantExpr = (0, typed_parser_1.seq)((0, typed_parser_1.oneOf)((0, token_1.expectIdentifier)('TRUE'), (0, token_1.expectIdentifier)('FALSE'), (0, token_1.expectIdentifier)('NULL'), (0, typed_parser_1.match)('[0-9]+(\\.[0-9]+)?([eE]-?[0-9]+)?'), (0, typed_parser_1.match)('\\.[0-9]+'), stringConstant), token_1._)((val, _ws) => ast_1.Expression.createConstant(val));
const parameterExpr = (0, typed_parser_1.seq)((0, token_1.symbolKeepWS)('$'), (0, typed_parser_1.int)('[0-9]+'), token_1._)((_$, index, _ws) => ast_1.Expression.createParameter(index));
const parenthesizedExpr = parenthesized((0, typed_parser_1.lazy)(() => expression));
const typeName = (0, typed_parser_1.seq)(token_1.identifier, (0, typed_parser_1.oneOf)((0, typed_parser_1.seqConst)('[]', (0, token_1.symbol)('['), (0, token_1.symbol)(']')), (0, typed_parser_1.seqConst)('', token_1._)))((id, arraySuffix) => id + arraySuffix);
const primaryExpr = (0, typed_parser_1.seq)((0, typed_parser_1.oneOf)(arraySubQueryExpr, caseExpr, (0, typed_parser_1.attempt)((0, special_functions_1.specialFunctionCall)((0, typed_parser_1.lazy)(() => primaryExpr))), columnRefOrFunctionCallExpr, constantExpr, parameterExpr, parenthesizedExpr), (0, utils_1.optional)((0, typed_parser_1.seq2)((0, token_1.symbol)('::'), typeName)))((expr, typeCast) => typeCast != null ? ast_1.Expression.createTypeCast(expr, typeCast) : expr);
function makeUnaryOp(oper, nextExpr) {
    return (0, typed_parser_1.seq)((0, typed_parser_1.many)(oper), nextExpr)((ops, next) => ops.length > 0
        ? ops.reduceRight((acc, op) => ast_1.Expression.createUnaryOp(op, acc), next)
        : next);
}
function makeBinaryOp(oper, nextExpr) {
    return (0, typed_parser_1.seq)(nextExpr, (0, typed_parser_1.many)((0, typed_parser_1.seq)(oper, nextExpr)((op, next) => ({ op, next }))))((first, rest) => rest.reduce((acc, val) => ast_1.Expression.createBinaryOp(acc, val.op, val.next), first));
}
const oneOfOperators = (...ops) => (0, typed_parser_1.oneOf)(...ops.map(token_1.operator));
const subscriptExpr = (0, typed_parser_1.seq)(primaryExpr, (0, typed_parser_1.many)((0, typed_parser_1.seq2)((0, token_1.symbol)('['), (0, typed_parser_1.lazy)(() => expression), (0, token_1.symbol)(']'))))((next, subs) => subs.reduce((acc, val) => ast_1.Expression.createBinaryOp(acc, '[]', val), next));
const unaryPlusMinus = makeUnaryOp(oneOfOperators('+', '-'), subscriptExpr);
const expExpr = makeBinaryOp((0, token_1.operator)('^'), unaryPlusMinus);
const mulDivModExpr = makeBinaryOp(oneOfOperators('*', '/', '%'), expExpr);
const addSubExpr = makeBinaryOp(oneOfOperators('+', '-'), mulDivModExpr);
const existsExpr = (0, typed_parser_1.seq)((0, token_1.reservedWord)('EXISTS'), parenthesized((0, typed_parser_1.lazy)(() => select)))((_exists, subquery) => ast_1.Expression.createExistsOp(subquery));
var OtherExprRhs;
(function (OtherExprRhs) {
    const in_ = (0, typed_parser_1.seq)((0, typed_parser_1.attempt)((0, typed_parser_1.oneOf)((0, token_1.reservedWord)('IN'), (0, typed_parser_1.seqConst)('NOT IN', (0, token_1.reservedWord)('NOT'), (0, token_1.reservedWord)('IN')))), parenthesized((0, typed_parser_1.lazy)(() => select)))((op, rhs) => ({ kind: 'InExprRhs', op, rhs }));
    const ternary = (0, typed_parser_1.seq)((0, typed_parser_1.oneOf)((0, typed_parser_1.attempt)((0, token_1.sepReserveds)('NOT BETWEEN SYMMETRIC')), (0, typed_parser_1.attempt)((0, token_1.sepReserveds)('BETWEEN SYMMETRIC')), (0, typed_parser_1.attempt)((0, token_1.sepReserveds)('NOT BETWEEN')), (0, typed_parser_1.attempt)((0, token_1.reservedWord)('BETWEEN'))), addSubExpr, (0, token_1.reservedWord)('AND'), addSubExpr)((op, rhs1, _and, rhs2) => ({ kind: 'TernaryExprRhs', op, rhs1, rhs2 }));
    const unarySuffix = (0, typed_parser_1.seqConst)({ kind: 'UnarySuffix', op: '!' }, (0, token_1.operator)('!'));
    const otherOp = (0, typed_parser_1.seq)((0, typed_parser_1.oneOf)((0, token_1.anyOperatorExcept)([
        '<',
        '>',
        '=',
        '<=',
        '>=',
        '<>',
        '+',
        '-',
        '*',
        '/',
        '%',
        '^',
    ]), (0, token_1.sepReserveds)('IS DISTINCT FROM'), (0, token_1.sepReserveds)('IS NOT DISTINCT FROM'), (0, token_1.reservedWord)('LIKE')), addSubExpr)((op, rhs) => ({ kind: 'OtherOpExprRhs', op, rhs }));
    OtherExprRhs.parser = (0, typed_parser_1.oneOf)(in_, ternary, unarySuffix, otherOp);
    function createExpression(lhs, rhs) {
        switch (rhs.kind) {
            case 'InExprRhs':
                return ast_1.Expression.createInOp(lhs, rhs.op, rhs.rhs);
            case 'TernaryExprRhs':
                return ast_1.Expression.createTernaryOp(lhs, rhs.op, rhs.rhs1, rhs.rhs2);
            case 'UnarySuffix':
                return ast_1.Expression.createUnaryOp(rhs.op, lhs);
            case 'OtherOpExprRhs':
                return ast_1.Expression.createBinaryOp(lhs, rhs.op, rhs.rhs);
        }
    }
    OtherExprRhs.createExpression = createExpression;
})(OtherExprRhs || (OtherExprRhs = {}));
const otherExpr = (0, typed_parser_1.seq)(addSubExpr, (0, typed_parser_1.many)(OtherExprRhs.parser))((first, rest) => rest.reduce((acc, val) => OtherExprRhs.createExpression(acc, val), first));
const existsOrOtherExpr = (0, typed_parser_1.oneOf)(existsExpr, otherExpr);
const comparisonExpr = makeBinaryOp(oneOfOperators('<', '>', '=', '<=', '>=', '<>'), existsOrOtherExpr);
const isExpr = (0, typed_parser_1.seq)(comparisonExpr, (0, utils_1.optional)((0, typed_parser_1.oneOf)((0, token_1.sepReserveds)('IS NULL'), (0, token_1.sepReserveds)('IS NOT NULL'), (0, token_1.reservedWord)('ISNULL'), (0, token_1.reservedWord)('NOTNULL'), (0, token_1.sepReserveds)('IS TRUE'), (0, token_1.sepReserveds)('IS NOT TRUE'), (0, token_1.sepReserveds)('IS FALSE'), (0, token_1.sepReserveds)('IS NOT FALSE'), (0, token_1.sepReserveds)('IS UNKNOWN'), (0, token_1.sepReserveds)('IS NOT UNKNOWN'))))((next, op) => (op ? ast_1.Expression.createUnaryOp(op, next) : next));
const notExpr = makeUnaryOp((0, token_1.reservedWord)('NOT'), isExpr);
const andExpr = makeBinaryOp((0, token_1.reservedWord)('AND'), notExpr);
const orExpr = makeBinaryOp((0, token_1.reservedWord)('OR'), andExpr);
const expression = orExpr;
// (name1, name2, ...)
const identifierList = parenthesized((0, typed_parser_1.sepBy1)((0, token_1.symbol)(','), token_1.identifier));
// WITH
const withQueries = (0, typed_parser_1.seq2)((0, token_1.reservedWord)('WITH'), (0, typed_parser_1.sepBy1)((0, token_1.symbol)(','), (0, typed_parser_1.seq)(token_1.identifier, (0, utils_1.optional)(identifierList), (0, token_1.reservedWord)('AS'), parenthesized((0, typed_parser_1.lazy)(() => statementParser)))((as, columns, _as, stmt) => ast_1.WithQuery.create(as, columns, stmt))));
const crossJoin = (0, typed_parser_1.seq)((0, token_1.sepReserveds)('CROSS JOIN'), (0, typed_parser_1.lazy)(() => tableExpression))((_cj, tableExpr) => ({ join: { kind: 'Cross' }, tableExpr }));
const qualifiedJoinType = (0, typed_parser_1.seq)((0, utils_1.optional)((0, typed_parser_1.oneOf)((0, token_1.reservedWord)('INNER'), (0, typed_parser_1.seq1)((0, typed_parser_1.oneOf)((0, token_1.reservedWord)('LEFT'), (0, token_1.reservedWord)('RIGHT'), (0, token_1.reservedWord)('FULL')), (0, utils_1.optional)((0, token_1.reservedWord)('OUTER'))))), (0, token_1.reservedWord)('JOIN'))((joinType, _join) => joinType || 'INNER');
const qualifiedJoin = (0, typed_parser_1.seq)(qualifiedJoinType, (0, typed_parser_1.lazy)(() => tableExpression), (0, typed_parser_1.oneOf)((0, typed_parser_1.seq)((0, token_1.reservedWord)('ON'), expression)((_on, expr) => ast_1.TableExpression.createJoinOn(expr)), (0, typed_parser_1.seq)((0, token_1.reservedWord)('USING'), identifierList)((_using, columns) => ast_1.TableExpression.createJoinUsing(columns))))((type, tableExpr, condition) => ({
    join: { kind: 'Qualified', type, condition },
    tableExpr,
}));
const naturalJoinType = (0, typed_parser_1.seq2)((0, token_1.reservedWord)('NATURAL'), qualifiedJoinType);
const naturalJoin = (0, typed_parser_1.seq)(naturalJoinType, (0, typed_parser_1.lazy)(() => tableExpression))((type, tableExpr) => ({ join: { kind: 'Natural', type }, tableExpr }));
const table = (0, typed_parser_1.seq)(tableRef, (0, utils_1.optional)(as))(ast_1.TableExpression.createTable);
function tableExprReducer(acc, joinSpec) {
    switch (joinSpec.join.kind) {
        case 'Cross':
            return ast_1.TableExpression.createCrossJoin(acc, joinSpec.tableExpr);
        case 'Qualified':
            return ast_1.TableExpression.createQualifiedJoin(acc, joinSpec.join.type, joinSpec.tableExpr, joinSpec.join.condition);
        case 'Natural':
            return ast_1.TableExpression.createQualifiedJoin(acc, joinSpec.join.type, joinSpec.tableExpr, ast_1.TableExpression.joinNatural);
    }
}
const tableExpression = (0, typed_parser_1.seq)((0, typed_parser_1.oneOf)((0, typed_parser_1.attempt)((0, typed_parser_1.seq2)((0, token_1.symbol)('('), (0, typed_parser_1.lazy)(() => tableExpression), (0, token_1.symbol)(')'))), (0, typed_parser_1.attempt)((0, typed_parser_1.seq)(parenthesized((0, typed_parser_1.lazy)(() => statementParser)), as)((stmt, as) => ast_1.TableExpression.createSubQuery(stmt, as))), table), (0, typed_parser_1.many)((0, typed_parser_1.oneOf)(crossJoin, qualifiedJoin, naturalJoin)))((lhs, rest) => (rest.length === 0 ? lhs : rest.reduce(tableExprReducer, lhs)));
const from = (0, typed_parser_1.seq)((0, token_1.reservedWord)('FROM'), tableExpression, (0, typed_parser_1.many)((0, typed_parser_1.seq2)((0, token_1.symbol)(','), tableExpression)))((_from, tableExpr, rest) => rest.length === 0
    ? tableExpr
    : // Implicit join equals to CROSS JOIN
        rest.reduce((acc, next) => ast_1.TableExpression.createCrossJoin(acc, next), tableExpr));
// WHERE
const where = (0, typed_parser_1.seq2)((0, token_1.reservedWord)('WHERE'), expression);
// GROUP BY & HAVING
const groupBy = (0, typed_parser_1.seq3)((0, token_1.reservedWord)('GROUP'), (0, token_1.reservedWord)('BY'), (0, typed_parser_1.sepBy1)((0, token_1.symbol)(','), expression));
const having = (0, typed_parser_1.seq2)((0, token_1.reservedWord)('HAVING'), expression);
// WINDOW
const window = (0, typed_parser_1.seq2)((0, token_1.reservedWord)('WINDOW'), (0, typed_parser_1.sepBy1)((0, token_1.symbol)(','), (0, typed_parser_1.seq)(token_1.identifier, (0, token_1.reservedWord)('AS'), parenthesized(windowDefinition))((name, _as, window) => ast_1.NamedWindowDefinition.create(name, window))));
// ORDER BY
const orderByOrder = (0, typed_parser_1.oneOf)((0, token_1.reservedWord)('ASC'), (0, token_1.reservedWord)('DESC'), (0, typed_parser_1.seq)((0, token_1.reservedWord)('USING'), token_1.anyOperator)((_using, op) => ['USING', op]));
const orderByNulls = (0, typed_parser_1.seq2)((0, token_1.reservedWord)('NULLS'), (0, typed_parser_1.oneOf)((0, token_1.reservedWord)('FIRST'), (0, token_1.reservedWord)('LAST')));
const orderByItem = (0, typed_parser_1.seq)(expression, (0, utils_1.optional)(orderByOrder), (0, utils_1.optional)(orderByNulls))((expr, order, nulls) => ast_1.OrderBy.create(expr, order, nulls));
const orderBy = (0, typed_parser_1.seq)((0, token_1.reservedWord)('ORDER'), (0, token_1.reservedWord)('BY'), (0, typed_parser_1.sepBy1)((0, token_1.symbol)(','), orderByItem))((_order, _by, list) => list);
// LIMIT
const limit = (0, typed_parser_1.seq)((0, token_1.reservedWord)('LIMIT'), (0, typed_parser_1.oneOf)((0, typed_parser_1.seqNull)((0, token_1.reservedWord)('ALL')), expression), (0, utils_1.optional)((0, typed_parser_1.seq2)((0, token_1.reservedWord)('OFFSET'), expression)))((_l, count, offset) => ast_1.Limit.create(count, offset));
// SELECT
const distinct = (0, typed_parser_1.oneOf)((0, token_1.reservedWord)('ALL'), (0, typed_parser_1.seq)((0, token_1.reservedWord)('DISTINCT'), (0, utils_1.optional)((0, typed_parser_1.seq2)((0, token_1.reservedWord)('ON'), parenthesized((0, typed_parser_1.sepBy1)((0, token_1.symbol)(','), expression)))))((_distinct, on) => (on ? on : 'DISTINCT')));
const allFields = (0, typed_parser_1.seqConst)(ast_1.SelectListItem.createAllFields(), (0, token_1.symbol)('*'));
const allTableFields = (0, typed_parser_1.seq)(token_1.identifier, (0, token_1.symbol)('.'), (0, token_1.symbol)('*'))((table, _p, _a) => ast_1.SelectListItem.createAllTableFields(table));
const selectListExpression = (0, typed_parser_1.seq)(expression, (0, utils_1.optional)(as))((expr, as) => ast_1.SelectListItem.createSelectListExpression(expr, as));
const selectListItem = (0, typed_parser_1.oneOf)(allFields, (0, typed_parser_1.attempt)(allTableFields), selectListExpression);
const selectList = (0, typed_parser_1.sepBy1)((0, token_1.symbol)(','), selectListItem);
const selectBody = (0, typed_parser_1.seq)((0, token_1.reservedWord)('SELECT'), (0, utils_1.optional)(distinct), selectList, (0, utils_1.optional)(from), (0, utils_1.optional)(where), (0, utils_1.optional)(groupBy), (0, utils_1.optional)(having), (0, utils_1.optional)(window))((_sel, distinct, list, from, where, groupBy, having, window) => ast_1.SelectBody.create(distinct || 'ALL', list, from, where, groupBy || [], having, window || []));
const selectSetOps = (0, typed_parser_1.many)((0, typed_parser_1.seq)((0, typed_parser_1.oneOf)((0, token_1.reservedWord)('UNION'), (0, token_1.reservedWord)('INTERSECT'), (0, token_1.reservedWord)('EXCEPT')), (0, utils_1.optional)((0, typed_parser_1.oneOf)((0, token_1.reservedWord)('ALL'), (0, token_1.reservedWord)('DISTINCT'))), selectBody)((op, duplicates, body) => ast_1.SelectOp.create(op, duplicates || 'DISTINCT', body)));
const select = (0, typed_parser_1.seq)((0, utils_1.optional)(withQueries), selectBody, selectSetOps, (0, utils_1.optional)(orderBy), (0, utils_1.optional)(limit))((withQueries, body, setOps, orderBy, limit) => ast_1.Select.create(withQueries || [], body, setOps, orderBy || [], limit));
// INSERT
const defaultValues = (0, typed_parser_1.seq)((0, token_1.reservedWord)('DEFAULT'), (0, token_1.reservedWord)('VALUES'))((_def, _val) => ast_1.Values.defaultValues);
const expressionValues = (0, typed_parser_1.seq)((0, token_1.reservedWord)('VALUES'), (0, typed_parser_1.sepBy)((0, token_1.symbol)(','), parenthesized((0, typed_parser_1.sepBy1)((0, token_1.symbol)(','), (0, typed_parser_1.oneOf)((0, typed_parser_1.seqNull)((0, token_1.reservedWord)('DEFAULT')), expression)))))((_val, values) => ast_1.Values.createExpressionValues(values));
const values = (0, typed_parser_1.oneOf)(defaultValues, expressionValues);
const onConstraint = (0, typed_parser_1.seqNull)((0, token_1.reservedWord)('ON'), (0, token_1.reservedWord)('CONSTRAINT'), token_1.identifier);
const conflictTarget = (0, typed_parser_1.oneOf)((0, typed_parser_1.seqNull)(identifierList), onConstraint);
const conflictAction = (0, typed_parser_1.seq2)((0, token_1.reservedWord)('DO'), (0, typed_parser_1.oneOf)((0, typed_parser_1.seqNull)((0, token_1.reservedWord)('NOTHING')), (0, typed_parser_1.seq2)((0, token_1.reservedWord)('UPDATE'), (0, typed_parser_1.lazy)(() => updateAssignments))));
const onConflict = (0, typed_parser_1.seq)((0, token_1.reservedWord)('ON'), (0, token_1.reservedWord)('CONFLICT'), (0, utils_1.optional)(conflictTarget), conflictAction)((_on, _conflict, _target, action) => action);
const returning = (0, typed_parser_1.seq2)((0, token_1.reservedWord)('RETURNING'), selectList);
const insertInto = (0, typed_parser_1.seq3)((0, token_1.reservedWord)('INSERT'), (0, token_1.reservedWord)('INTO'), tableRef);
const insert = (0, typed_parser_1.seq)((0, utils_1.optional)(withQueries), insertInto, (0, utils_1.optional)(reqAs), (0, utils_1.optional)(identifierList), (0, typed_parser_1.oneOf)(values, (0, typed_parser_1.lazy)(() => select)), (0, utils_1.optional)(onConflict), (0, utils_1.optional)(returning))((withQueries, table, as, columns, values, onConflict, returning) => ast_1.Insert.create(withQueries || [], table, as, columns || [], values, onConflict || [], returning || []));
// UPDATE
const updateAssignments = (0, typed_parser_1.seq2)((0, token_1.reservedWord)('SET'), (0, typed_parser_1.sepBy1)((0, token_1.symbol)(','), (0, typed_parser_1.seq)(token_1.identifier, (0, token_1.symbol)('='), expression)((columnName, _eq, value) => ({ columnName, value }))));
const updateTable = (0, typed_parser_1.seq2)((0, token_1.reservedWord)('UPDATE'), tableRef);
const update = (0, typed_parser_1.seq)((0, utils_1.optional)(withQueries), updateTable, (0, utils_1.optional)(reqAs), updateAssignments, (0, utils_1.optional)(from), (0, utils_1.optional)(where), (0, utils_1.optional)(returning))((withQueries, table, as, updates, from, where, returning) => ast_1.Update.create(withQueries || [], table, as, updates, from, where, returning || []));
// DELETE
const deleteFrom = (0, typed_parser_1.seq3)((0, token_1.reservedWord)('DELETE'), (0, token_1.reservedWord)('FROM'), tableRef);
const delete_ = (0, typed_parser_1.seq)(deleteFrom, (0, utils_1.optional)(reqAs), (0, utils_1.optional)(where), (0, utils_1.optional)(returning))((table, as, where, returning) => ast_1.Delete.create(table, as, where, returning || []));
// parse
const statementParser = (0, typed_parser_1.seq1)((0, typed_parser_1.oneOf)(select, insert, update, delete_), (0, utils_1.optional)((0, token_1.symbol)(';')));
const topLevelParser = (0, typed_parser_1.seq2)(token_1._, statementParser, typed_parser_1.end);
function parse(source) {
    return (0, Either_1.tryCatch)(() => (0, typed_parser_1.run)(topLevelParser, source), (e) => e.explain());
}
exports.parse = parse;
