"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.specialFunctionCall = void 0;
const typed_parser_1 = require("../typed-parser");
const ast_1 = require("../ast");
const token_1 = require("./token");
const utils_1 = require("./utils");
// We need to take the expression parser as an argument to avoid cyclic
// imports
function specialFunctionCall(primaryExpr) {
    function specialFunctionParser(funcName, argsParser) {
        return (0, typed_parser_1.seq)((0, token_1.expectIdentifier)(funcName), (0, utils_1.parenthesized)(argsParser))((_, argList) => ast_1.Expression.createFunctionCall(null, funcName, argList, null, null));
    }
    const overlayFunction = specialFunctionParser('overlay', (0, typed_parser_1.seq)(primaryExpr, (0, token_1.expectIdentifier)('placing'), primaryExpr, (0, token_1.expectIdentifier)('from'), primaryExpr, (0, token_1.expectIdentifier)('for'), primaryExpr)((a1, _placing, a2, _from, a3, _for, a4) => [a1, a2, a3, a4]));
    const positionFunction = specialFunctionParser('position', (0, typed_parser_1.seq)(primaryExpr, (0, token_1.expectIdentifier)('in'), primaryExpr)((a1, _in, a2) => [a1, a2]));
    const substringFunction = specialFunctionParser('substring', (0, typed_parser_1.seq)(primaryExpr, (0, token_1.expectIdentifier)('from'), primaryExpr, (0, utils_1.optional)((0, typed_parser_1.seq2)((0, token_1.expectIdentifier)('for'), primaryExpr)))((a1, _from, a2, a3) => (a3 ? [a1, a2, a3] : [a1, a2])));
    const trimDirection = (0, typed_parser_1.oneOf)((0, token_1.expectIdentifier)('leading'), (0, token_1.expectIdentifier)('trailing'), (0, token_1.expectIdentifier)('both'));
    function trimArgs(dir, chrs, str) {
        return [
            ast_1.Expression.createConstant(dir || 'both'),
            chrs || ast_1.Expression.createConstant(''),
            str,
        ];
    }
    // trim([leading | trailing | both] from string [, characters] )
    // trim([leading | trailing | both] characters from string)
    // trim([leading | trailing | both] string [, characters] )
    const trimFunction = specialFunctionParser('trim', (0, typed_parser_1.seq)((0, utils_1.optional)(trimDirection), (0, typed_parser_1.oneOf)((0, typed_parser_1.seq)((0, token_1.expectIdentifier)('from'), primaryExpr, (0, utils_1.optional)((0, typed_parser_1.seq2)((0, token_1.symbol)(','), primaryExpr)))((_from, str, chars) => [str, chars]), (0, typed_parser_1.oneOf)((0, typed_parser_1.attempt)((0, typed_parser_1.seq)(primaryExpr, (0, token_1.expectIdentifier)('from'), primaryExpr)((chrs, _from, str) => [str, chrs])), (0, typed_parser_1.seq)(primaryExpr, (0, utils_1.optional)((0, typed_parser_1.seq2)((0, token_1.symbol)(','), primaryExpr)))((str, chrs) => [str, chrs]))))((dir, [str, chrs]) => trimArgs(dir, chrs, str)));
    return (0, typed_parser_1.oneOf)(overlayFunction, positionFunction, substringFunction, trimFunction);
}
exports.specialFunctionCall = specialFunctionCall;
