"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.schemaClient = void 0;
const Either = require("fp-ts/lib/Either");
const Task = require("fp-ts/lib/Task");
const function_1 = require("fp-ts/lib/function");
const sql = require("./sql");
const const_utils_1 = require("./const-utils");
const func_utils_1 = require("./func-utils");
function schemaClient(postgresClient) {
    const getTable = (schemaName, tableName) => () => __awaiter(this, void 0, void 0, function* () {
        const result = yield sql.tableColumns(postgresClient, {
            schemaName: schemaName || 'public',
            tableName,
        });
        if (result.length === 0) {
            return Either.left(`No such table: ${fullTableName(schemaName, tableName)}`);
        }
        return Either.right({
            name: tableName,
            columns: result
                .filter((col) => col.attisdropped === false)
                .map((col) => ({
                hidden: col.attnum < 0,
                name: col.attname,
                nullable: !col.attnotnull,
                type: col.atttypid,
            })),
        });
    });
    const getEnums = (0, func_utils_1.asyncCached)(() => __awaiter(this, void 0, void 0, function* () {
        return (yield sql.enums(postgresClient)).map((row) => ({
            oid: row.oid,
            name: row.typname,
            labels: row.labels,
        }));
    }));
    const getArrayTypes = (0, func_utils_1.asyncCached)(() => __awaiter(this, void 0, void 0, function* () {
        return (yield sql.arrayTypes(postgresClient)).map((row) => ({
            oid: row.oid,
            elemType: row.typelem,
        }));
    }));
    const getFunctions = (0, func_utils_1.asyncCached)(() => __awaiter(this, void 0, void 0, function* () {
        return (yield sql.functions(postgresClient)).map((row) => ({
            schema: row.nspname,
            name: row.proname,
            nullSafety: 'unsafe',
        }));
    }));
    const functionNullSafety = (schemaName, functionName) => {
        const builtin = schemaName === null ? (0, const_utils_1.builtinFunctionNullSafety)(functionName) : null;
        if (builtin !== null)
            return Task.of(builtin);
        return (0, function_1.pipe)(getFunctions, Task.map((functions) => {
            const fn = functions.find((fn) => fn.name === functionName &&
                (schemaName === null || fn.schema === schemaName));
            if (fn)
                return fn.nullSafety;
            return null;
        }));
    };
    return { getTable, getEnums, getArrayTypes, functionNullSafety };
}
exports.schemaClient = schemaClient;
function fullTableName(schemaName, tableName) {
    return (schemaName ? schemaName + '.' : '') + tableName;
}
